/*
 * Commanding.c
 *
 *  Created on: 8. nov. 2018
 *      Author: sb
 */

#include "driverlib/driverlib.h"
#include "Commanding.h"



typedef enum
{
    Delimiter = 0,
    Data1,
    Data2

} ProtocolParseState;

static ProtocolParseState _state;
static uint16_t _rxBuffer;
static CommandHandler _handler;

void Commanding_Initialize(CommandHandler handler)
{
    //Init state
    _state = Delimiter;
    _rxBuffer = 0;
    _handler = handler;

    /* Select Port 2 (UCA0 Pins)
    * Set Pin 1, 0 to input Primary Module Function, LFXT.
    */
    GPIO_setAsPeripheralModuleFunctionInputPin(
        GPIO_PORT_P2,
        GPIO_PIN0 + GPIO_PIN1,
        GPIO_SECONDARY_MODULE_FUNCTION
    );

    // Configure UART
    EUSCI_A_UART_initParam param = {0};
    param.selectClockSource = EUSCI_A_UART_CLOCKSOURCE_SMCLK;
    param.clockPrescalar = 78;
    param.firstModReg = 2;
    param.secondModReg = 0x00;
    param.parity = EUSCI_A_UART_NO_PARITY;
    param.msborLsbFirst = EUSCI_A_UART_LSB_FIRST;
    param.numberofStopBits = EUSCI_A_UART_ONE_STOP_BIT;
    param.uartMode = EUSCI_A_UART_MODE;
    param.overSampling = EUSCI_A_UART_OVERSAMPLING_BAUDRATE_GENERATION;

    if (STATUS_FAIL == EUSCI_A_UART_init(EUSCI_A0_BASE, &param)) {
        return;
    }

    EUSCI_A_UART_enable(EUSCI_A0_BASE);

    EUSCI_A_UART_clearInterrupt(EUSCI_A0_BASE, EUSCI_A_UART_RECEIVE_INTERRUPT);

    // Enable USCI_A0 RX interrupt
    EUSCI_A_UART_enableInterrupt(EUSCI_A0_BASE, EUSCI_A_UART_RECEIVE_INTERRUPT); // Enable interrupt
}

#pragma vector=USCI_A0_VECTOR
__interrupt void EUSCI_A0_ISR(void)
{
    uint8_t data = 0;

  switch(__even_in_range(UCA0IV,USCI_UART_UCTXCPTIFG))
  {
    case USCI_NONE: break;
    case USCI_UART_UCRXIFG:
        data = EUSCI_A_UART_receiveData(EUSCI_A0_BASE);

        switch(_state)
        {
            case Delimiter:
                if((data & BIT7) > 0) //Delimter bit set
                {
                    _rxBuffer = (data << 8);
                    _state++; //Found sync bit
                }

                break;
            case Data1:
                _rxBuffer |= data;
                _state++;
                break;
            case Data2:
            {
                //Delimiter high indicates a start delimiter
                //| Delim.  (1 bit) | CommandId (5 bit) | Data1 (2 bit)|
                //| Delim.  (1 bit) | Data2 (7 bit) |
                //| Delim.  (1 bit) | Data3 (7 bit) |

                uint8_t commandId = (_rxBuffer >> 10) & 0x1F; //5 bit
                uint16_t value = ((_rxBuffer >> 8) & 0x03) << 14;
                value |= (_rxBuffer & 0x7F) << 7;
                value |= (data & 0x7F);

                if(_handler)
                    _handler(commandId, value);

                _state = Delimiter;
                break;
            }
        }

        EUSCI_A_UART_transmitData(EUSCI_A0_BASE, data); //ECHO
      break;
    case USCI_UART_UCTXIFG: break;
    case USCI_UART_UCSTTIFG: break;
    case USCI_UART_UCTXCPTIFG: break;
  }
}
